@@ -1,6 +1,6 @@
 /* h316_cpu.c: Honeywell 316/516 CPU simulator
 
-   Copyright (c) 1999-2004, Robert M. Supnik
+   Copyright (c) 1999-2005, Robert M. Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -25,6 +25,8 @@
 
    cpu		H316/H516 CPU
 
+   15-Feb-05	RMS	Added start button interrupt
+   01-Dec-04	RMS	Fixed bug in DIV
    06-Nov-04	RMS	Added =n to SHOW HISTORY
    04-Jan-04	RMS	Removed unnecessary compare
    31-Dec-03	RMS	Fixed bug in cpu_set_hist
@@ -307,6 +309,7 @@ REG cpu_reg[] = {
 	{ FLDATA (SS4, ss[3], 0) },
 	{ FLDATA (ION, dev_int, INT_V_ON) },
 	{ FLDATA (INODEF, dev_int, INT_V_NODEF) },
+	{ FLDATA (START, dev_int, INT_V_START) },
 	{ ORDATA (DEVINT, dev_int, 16), REG_RO },
 	{ ORDATA (DEVENB, dev_enb, 16), REG_RO },
 	{ ORDATA (CHREQ, chan_req, DMA_MAX + DMC_MAX) },
@@ -442,7 +445,7 @@ if (chan_req) {						/* channel request? */
 
 /* Interrupts */
 
-if ((dev_int & (INT_PENDING | dev_enb)) > INT_PENDING) {	/* int req? */
+if ((dev_int & (INT_PEND|INT_NMI|dev_enb)) > INT_PEND) {/* int req? */
 	pme = ext;					/* save extend */
 	if (cpu_unit.flags & UNIT_EXT) ext = 1;		/* ext opt? extend on */
 	dev_int = dev_int & ~INT_ON;			/* intr off */
@@ -459,6 +462,7 @@ else {	if (sim_brk_summ &&
 	PC = NEWA (Y, Y + 1);				/* incr PC */
 	dev_int = dev_int | INT_NODEF;  }
 
+dev_int = dev_int & ~INT_START;				/* clr start button int */
 sim_interval = sim_interval - 1;
 if (hst_lnt) {						/* instr hist? */
 	hst_p = (hst_p + 1);				/* next entry */
@@ -496,11 +500,10 @@ case 003: case 023: case 043: case 063:			/* ANA */
 
 case 004: case 024: case 044: case 064:			/* STA */
 	if (reason = Ea (MB, &Y)) break;		/* eff addr */
+	Write (Y, AR);					/* store A */
 	if (dp) {					/* double prec? */
-	    if ((Y & 1) == 0) Write (Y, AR);		/* if even, store A */
 	    Write (Y | 1, BR);				/* store B */
 	    sc = 0;  }
-	else Write (Y, AR);				/* no, store word */
 	break;
 
 case 005: case 025: case 045: case 065:			/* ERA */
@@ -585,7 +588,7 @@ case 017: case 037: case 057: case 077:			/* DIV */
 	    if (reason = Ea (MB, &Y)) break;		/* eff addr */
 	    t2 = SEXT (Read (Y));			/* divr */
 	    if (t2) {					/* divr != 0? */
-		t1 = GETDBL_S (AR, BR);			/* get A'B */
+		t1 = GETDBL_S (SEXT (AR), BR);		/* get A'B signed */
 		BR = (t1 % t2) & DMASK;			/* remainder */
 		t1 = t1 / t2;				/* quotient */
 		AR = t1 & DMASK;
@@ -934,8 +937,8 @@ int32 Add16 (int32 v1, int32 v2)
 {
 int32 r = v1 + v2;
 
-C = 0;
 if (((v1 ^ ~v2) & (v1 ^ r)) & SIGN) C = 1;
+else C = 0;
 return (r & DMASK);
 }
 
@@ -943,8 +946,8 @@ int32 Add31 (int32 v1, int32 v2)
 {
 int32 r = v1 + v2;
 
-C = 0;
-if (((v1 ^ ~v2) & (v1 ^ r)) & (1u << 30)) C = 1;
+if (((v1 ^ ~v2) & (v1 ^ r)) & DP_SIGN) C = 1;
+else C = 0;
 return r;
 }
 
@@ -1129,7 +1132,7 @@ saved_AR = saved_BR = saved_XR = 0;
 C = 0;
 dp = 0;
 ext = pme = extoff_pending = 0;
-dev_int = dev_int & ~INT_PENDING;
+dev_int = dev_int & ~(INT_PEND|INT_NMI);
 dev_enb = 0;
 for (i = 0; i < DMA_MAX; i++) dma_ad[i] = dma_wc[i] = dma_eor[i] = 0;
 chan_req = 0;
@@ -1196,7 +1199,7 @@ if (cptr == NULL) return SCPE_ARG;
 newmax = get_uint (cptr, 10, DMA_MAX, &r);		/* get new max */
 if ((r != SCPE_OK) || (newmax == dma_nch)) return r;	/* err or no chg? */
 dma_nch = newmax;					/* set new max */
-for (i = newmax; i <DMA_MAX; i++) {			/* reset chan */
+for (i = newmax; i < DMA_MAX; i++) {			/* reset chan */
 	dma_ad[i] = dma_wc[i] = dma_eor[i] = 0;
 	chan_req = chan_req & ~(1 << i);  }
 return SCPE_OK;
