@@ -92,10 +92,6 @@ t_stat stat;
 idx = (pa & IOPAGEMASK) >> 1;
 if (iodispR[idx]) {
 	stat = iodispR[idx] (data, pa, access);
-	if (sim_log &&
-	   (((pa >= IOBA_RQ) && (pa < IOBA_SUP)) ||
-	    ((pa >= IOBA_TU) && (pa < IOBA_SR))))
-	    fprintf (sim_log, ">>IORD: addr=%o, data=%o\n", pa, *data);
 	trap_req = calc_ints (ipl, trap_req);
 	return stat;  }
 return SCPE_NXM;
@@ -108,10 +104,6 @@ t_stat stat;
 
 idx = (pa & IOPAGEMASK) >> 1;
 if (iodispW[idx]) {
-	if (sim_log &&
-	   (((pa >= IOBA_RQ) && (pa < IOBA_SUP)) ||
-	    ((pa >= IOBA_TU) && (pa < IOBA_SR))))
-	    fprintf (sim_log, ">>IOWR: addr=%o, data=%o\n", pa, data);
 	stat = iodispW[idx] (data, pa, access);
 	trap_req = calc_ints (ipl, trap_req);
 	return stat;  }
