@@ -1,6 +1,6 @@
 /* pdp11_rq.c: MSCP disk controller simulator
 
-   Copyright (c) 2002-2004, Robert M Supnik
+   Copyright (c) 2002-2005, Robert M Supnik
    Derived from work by Stephen F. Shirron
 
    Permission is hereby granted, free of charge, to any person obtaining a
@@ -26,6 +26,13 @@
 
    rq		RQDX3 disk controller
 
+   17-Jan-05	RMS	Added more RA and RD disks
+   31-Oct-04	RMS	Added -L switch (LBNs) to RAUSER size specification
+   01-Oct-04	RMS	Revised Unibus interface
+			Changed to identify as UDA50 in Unibus configurations
+			Changed width to be 16b in all configurations
+			Changed default timing for VAX
+   24-Jul-04	RMS	VAX controllers luns start with 0 (from Andreas Cejna)
    05-Feb-04	RMS	Revised for file I/O library
    25-Jan-04	RMS	Revised for device debug support
    12-Jan-04	RMS	Fixed bug in interrupt control (found by Tom Evans)
@@ -63,18 +70,18 @@
 
 #elif defined (VM_VAX)					/* VAX version */
 #include "vax_defs.h"
-#define RQ_AINC		4
-#define RQ_WID		32
-extern int32 int_req[IPL_HLVL];
-extern int32 int_vec[IPL_HLVL][32];
+#define RQ_QTIME	100
+#define RQ_XTIME	200
+#define OLDPC		fault_PC
+extern int32 fault_PC;
 
 #else							/* PDP-11 version */
 #include "pdp11_defs.h"
-#define RQ_AINC		2
-#define RQ_WID		16
-extern int32 cpu_18b, cpu_ubm;
-extern int32 int_req[IPL_HLVL];
-extern int32 int_vec[IPL_HLVL][32];
+#define RQ_QTIME	200
+#define RQ_XTIME	500
+#define OLDPC		MMR2
+extern int32 MMR2;
+extern int32 cpu_opt;
 #endif
 
 #if !defined (RQ_NUMCT)
@@ -97,8 +104,12 @@ extern int32 int_vec[IPL_HLVL][32];
 #define RQ_SH_UN	010				/* show unit q's */
 
 #define RQ_CLASS	1				/* RQ class */
-#define RQ_UQPM		19				/* UQ port model */
-#define RQ_MODEL	19				/* MSCP ctrl model */
+#define RQU_UQPM	6				/* UB port model */
+#define RQQ_UQPM	19				/* QB port model */
+#define RQ_UQPM		(UNIBUS? RQU_UQPM: RQQ_UQPM)
+#define RQU_MODEL	6				/* UB MSCP ctrl model */
+#define RQQ_MODEL	19				/* QB MSCP ctrl model */
+#define RQ_MODEL	(UNIBUS? RQU_MODEL: RQQ_MODEL)
 #define RQ_HVER		1				/* hardware version */
 #define RQ_SVER		3				/* software version */
 #define RQ_DHTMO	60				/* def host timeout */
@@ -168,7 +179,7 @@ struct rqpkt {
 #define PUTP32(p,w,x)	cp->pak[p].d[w] = (x) & 0xFFFF; \
 			cp->pak[p].d[(w)+1] = ((x) >> 16) & 0xFFFF
 
-/* Disk formats.  An RQDX3 consists of the following regions:
+/* Disk formats.  An RQDX3 disk consists of the following regions:
 
    XBNs		Extended blocks - contain information about disk format,
 		also holds track being reformatted during bad block repl.
@@ -193,7 +204,7 @@ struct rqpkt {
 #define RCT_ENTB	128				/* entries/blk */
 #define RCT_END		0x80000000			/* marks RCT end */
 
-/* The RQDX3 supports multiple disk drive types:
+/* The RQDX3 supports multiple disk drive types (x = not implemented):
 
    type	sec	surf	cyl	tpg	gpc	RCT	LBNs
 	
@@ -202,17 +213,23 @@ struct rqpkt {
    RD51	18	4	306	4	1	36*4	21600
    RD31	17	4	615	4	1	3*8	41560
    RD52	17	8	512	8	1	4*8	60480
+x  RD32	17	6	820	?	?	?	83236
+x  RD33	17	7	1170	?	?	?	138565
    RD53	17	7	1024	7	1	5*8	138672
    RD54	17	15	1225	15	1	7*8	311200
 
    The simulator also supports larger drives that only existed
-   on SDI controllers.  XBN, DBN, RCTS and RCTC are not known
-   for the SDI drives and are not used by the simulator:
+   on SDI controllers.
 
-   RA82	57	15	1435	15	1	?*8	1216665
-   RA72	51	20	1921?	20	1	?*8	1953300
-   RA90	69	13	2656	13	1	?*8	2376153
-   RA92	73	13	3101	13	1	?*8	2940951
+   RA60	42(+1)	6	1600	6	1	1008	400176
+x  RA70	33(+1)	11	1507+	11	1	?	547041
+   RA81	51(+1)	14	1258	14	1	2856	891072
+   RA82	57(+1)	15	1435	15	1	3420	1216665
+   RA71 51(+1)	14	1921	14	1	1428	1367310		
+   RA72	51(+1)	20	1921	20	1	2040	1953300
+   RA90	69(+1)	13	2656	13	1	1794	2376153
+   RA92	73(+1)	13	3101	13	1	949	2940951
+x  RA73 70(+1)	21	2667+	21	1	?	3920490
 
    Each drive can be a different type.  The drive field in the
    unit flags specified the drive type and thus, indirectly,
@@ -341,11 +358,11 @@ struct rqpkt {
 #define RA82_CYL	1435				/* 0-1422 user */
 #define RA82_TPG	RA82_SURF
 #define RA82_GPC	1
-#define RA82_XBN	3420				/* cyl 1427-1430 */
-#define RA82_DBN	3420				/* cyl 1431-1434 */
+#define RA82_XBN	3480				/* cyl 1427-1430 */
+#define RA82_DBN	3480				/* cyl 1431-1434 */
 #define RA82_LBN	1216665				/* 57*15*1423 */
-#define RA82_RCTS	400				/* cyl 1423-1426 */
-#define RA82_RCTC	8
+#define RA82_RCTS	3420				/* cyl 1423-1426 */
+#define RA82_RCTC	1
 #define RA82_RBN	21345				/* 1 *15*1423 */
 #define RA82_MOD	11
 #define RA82_MED	0x25641052
@@ -373,11 +390,11 @@ struct rqpkt {
 #define RA72_CYL	1921				/* 0-1914 user */
 #define RA72_TPG	RA72_SURF
 #define RA72_GPC	1
-#define RA72_XBN	2040				/* cyl 1917-1918? */
-#define RA72_DBN	2040				/* cyl 1920-1921? */
+#define RA72_XBN	2080				/* cyl 1917-1918? */
+#define RA72_DBN	2080				/* cyl 1920-1921? */
 #define RA72_LBN	1953300				/* 51*20*1915 */
-#define RA72_RCTS	400				/* cyl 1915-1916? */
-#define RA72_RCTC	5				/* ? */
+#define RA72_RCTS	2040				/* cyl 1915-1916? */
+#define RA72_RCTC	1
 #define RA72_RBN	38300				/* 1 *20*1915 */
 #define RA72_MOD	37
 #define RA72_MED	0x25641048
@@ -389,11 +406,11 @@ struct rqpkt {
 #define RA90_CYL	2656				/* 0-2648 user */
 #define RA90_TPG	RA90_SURF
 #define RA90_GPC	1
-#define RA90_XBN	1794				/* cyl 2651-2652? */
-#define RA90_DBN	1794				/* cyl 2653-2654? */
+#define RA90_XBN	1820				/* cyl 2651-2652? */
+#define RA90_DBN	1820				/* cyl 2653-2654? */
 #define RA90_LBN	2376153				/* 69*13*2649 */
-#define RA90_RCTS	400				/* cyl 2649-2650? */
-#define RA90_RCTC	6				/* ? */
+#define RA90_RCTS	1794				/* cyl 2649-2650? */
+#define RA90_RCTC	1
 #define RA90_RBN	34437				/* 1 *13*2649 */
 #define RA90_MOD	19
 #define RA90_MED	0x2564105A
@@ -406,33 +423,81 @@ struct rqpkt {
 #define RA92_TPG	RA92_SURF
 #define RA92_GPC	1
 #define RA92_XBN	174				/* cyl 3100? */
-#define RA92_DBN	775
+#define RA92_DBN	788
 #define RA92_LBN	2940951				/* 73*13*3099 */
-#define RA92_RCTS	316				/* cyl 3099? */
-#define RA92_RCTC	3				/* ? */
+#define RA92_RCTS	949				/* cyl 3099? */
+#define RA92_RCTC	1
 #define RA92_RBN	40287				/* 1 *13*3099 */
 #define RA92_MOD	29
 #define RA92_MED	0x2564105C
 #define RA92_FLGS	RQDF_SDI
 
 #define RA8U_DTYPE	12				/* user defined */
-#define RA8U_SECT	57				/* +1 spare/track */
+#define RA8U_SECT	57				/* from RA82 */
 #define RA8U_SURF	15
-#define RA8U_CYL	1435				/* 0-1422 user */
+#define RA8U_CYL	1435				/* from RA82 */
 #define RA8U_TPG	RA8U_SURF
 #define RA8U_GPC	1
-#define RA8U_XBN	3420				/* cyl 1427-1430 */
-#define RA8U_DBN	3420				/* cyl 1431-1434 */
-#define RA8U_LBN	1216665				/* 57*15*1423 */
-#define RA8U_RCTS	400				/* cyl 1423-1426 */
+#define RA8U_XBN	0
+#define RA8U_DBN	0
+#define RA8U_LBN	1216665				/* from RA82 */
+#define RA8U_RCTS	400
 #define RA8U_RCTC	8
-#define RA8U_RBN	21345				/* 1 *15*1423 */
+#define RA8U_RBN	21345
 #define RA8U_MOD	11				/* RA82 */
 #define RA8U_MED	0x25641052			/* RA82 */
 #define RA8U_FLGS	RQDF_SDI
-#define RA8U_MINC	5				/* min cap MB */
-#define RA8U_MAXC	2000				/* max cap MB */
-#define RA8U_EMAXC	1000000				/* ext max cap */
+#define RA8U_MINC	10000				/* min cap LBNs */
+#define RA8U_MAXC	4000000				/* max cap LBNs */
+#define RA8U_EMAXC	2000000000			/* ext max cap */
+
+#define RA60_DTYPE	13				/* SDI drive */
+#define RA60_SECT	42				/* +1 spare/track */
+#define RA60_SURF	6
+#define RA60_CYL	1600				/* 0-1587 user */
+#define RA60_TPG	RA60_SURF
+#define RA60_GPC	1
+#define RA60_XBN	1032				/* cyl 1592-1595 */
+#define RA60_DBN	1032				/* cyl 1596-1599 */
+#define RA60_LBN	400176				/* 42*6*1588 */
+#define RA60_RCTS	1008				/* cyl 1588-1591 */
+#define RA60_RCTC	1
+#define RA60_RBN	9528				/* 1 *6*1588 */
+#define RA60_MOD	4
+#define RA60_MED	0x22A4103C
+#define RA60_FLGS	(RQDF_RMV | RQDF_SDI)
+
+#define RA81_DTYPE	14				/* SDI drive */
+#define RA81_SECT	51				/* +1 spare/track */
+#define RA81_SURF	14
+#define RA81_CYL	1258				/* 0-1247 user */
+#define RA81_TPG	RA81_SURF
+#define RA81_GPC	1
+#define RA81_XBN	2436				/* cyl 1252-1254? */
+#define RA81_DBN	2436				/* cyl 1255-1256? */
+#define RA81_LBN	891072				/* 51*14*1248 */
+#define RA81_RCTS	2856				/* cyl 1248-1251? */
+#define RA81_RCTC	1
+#define RA81_RBN	17472				/* 1 *14*1248 */
+#define RA81_MOD	5
+#define RA81_MED	0x25641051
+#define RA81_FLGS	RQDF_SDI
+
+#define RA71_DTYPE	15				/* SDI drive */
+#define RA71_SECT	51				/* +1 spare/track */
+#define RA71_SURF	14
+#define RA71_CYL	1921				/* 0-1914 user */
+#define RA71_TPG	RA71_SURF
+#define RA71_GPC	1
+#define RA71_XBN	1456				/* cyl 1917-1918? */
+#define RA71_DBN	1456				/* cyl 1919-1920? */
+#define RA71_LBN	1367310				/* 51*14*1915 */
+#define RA71_RCTS	1428				/* cyl 1915-1916? */
+#define RA71_RCTC	1
+#define RA71_RBN	26810				/* 1 *14*1915 */
+#define RA71_MOD	40
+#define RA71_MED	0x25641047
+#define RA71_FLGS	RQDF_SDI
 
 struct drvtyp {
 	int32	sect;					/* sectors */
@@ -466,19 +531,22 @@ static struct drvtyp drv_tab[] = {
 	{ RQ_DRV (RD54), "RD54" }, { RQ_DRV (RA82), "RA82" },
 	{ RQ_DRV (RRD40), "RRD40" }, { RQ_DRV (RA72), "RA72" },
 	{ RQ_DRV (RA90), "RA90" }, { RQ_DRV (RA92), "RA92" },
-	{ RQ_DRV (RA8U), "RAUSER" }, { 0 }  };
+	{ RQ_DRV (RA8U), "RAUSER" }, { RQ_DRV (RA60), "RA60" },
+	{ RQ_DRV (RA81), "RA81" }, { RQ_DRV (RA71), "RA71" },
+	{ 0 }  };
 
 extern int32 int_req[IPL_HLVL];
 extern int32 tmr_poll, clk_tps;
 extern UNIT cpu_unit;
 extern FILE *sim_deb;
 extern uint32 sim_taddr_64;
+extern int32 sim_switches;
 
 uint16 *rqxb = NULL;					/* xfer buffer */
 int32 rq_itime = 200;					/* init time, except */
 int32 rq_itime4 = 10;					/* stage 4 */
-int32 rq_qtime = 200;					/* queue time */
-int32 rq_xtime = 500;					/* transfer time */
+int32 rq_qtime = RQ_QTIME;				/* queue time */
+int32 rq_xtime = RQ_XTIME;				/* transfer time */
 
 struct mscp_con {
 	uint32		cnum;				/* ctrl number */
@@ -655,12 +723,18 @@ MTAB rq_mod[] = {
 		&rq_set_type, NULL, NULL },
 	{ MTAB_XTD | MTAB_VUN, RD54_DTYPE, NULL, "RD54",
 		&rq_set_type, NULL, NULL },
+	{ MTAB_XTD | MTAB_VUN, RA60_DTYPE, NULL, "RA60",
+		&rq_set_type, NULL, NULL },
+	{ MTAB_XTD | MTAB_VUN, RA81_DTYPE, NULL, "RA81",
+		&rq_set_type, NULL, NULL },
 	{ MTAB_XTD | MTAB_VUN, RA82_DTYPE, NULL, "RA82",
 		&rq_set_type, NULL, NULL },
 	{ MTAB_XTD | MTAB_VUN, RRD40_DTYPE, NULL, "RRD40",
 		&rq_set_type, NULL, NULL },
 	{ MTAB_XTD | MTAB_VUN, RRD40_DTYPE, NULL, "CDROM",
 		&rq_set_type, NULL, NULL },
+	{ MTAB_XTD | MTAB_VUN, RA71_DTYPE, NULL, "RA71",
+		&rq_set_type, NULL, NULL },
 	{ MTAB_XTD | MTAB_VUN, RA72_DTYPE, NULL, "RA72",
 		&rq_set_type, NULL, NULL },
 	{ MTAB_XTD | MTAB_VUN, RA90_DTYPE, NULL, "RA90",
@@ -686,7 +760,7 @@ MTAB rq_mod[] = {
 
 DEVICE rq_dev = {
 	"RQ", rq_unit, rq_reg, rq_mod,
-	RQ_NUMDR + 2, DEV_RDX, 31, RQ_AINC, DEV_RDX, RQ_WID,
+	RQ_NUMDR + 2, DEV_RDX, 31, 2, DEV_RDX, 16,
 	NULL, NULL, &rq_reset,
 	&rq_boot, &rq_attach, &rq_detach,
 	&rq_dib, DEV_FLTA | DEV_DISABLE | DEV_UBUS | DEV_QBUS | DEV_DEBUG };
@@ -750,7 +824,7 @@ REG rqb_reg[] = {
 
 DEVICE rqb_dev = {
 	"RQB", rqb_unit, rqb_reg, rq_mod,
-	RQ_NUMDR + 2, DEV_RDX, 31, RQ_AINC, DEV_RDX, RQ_WID,
+	RQ_NUMDR + 2, DEV_RDX, 31, 2, DEV_RDX, 16,
 	NULL, NULL, &rq_reset,
 	&rq_boot, &rq_attach, &rq_detach,
 	&rqb_dib, DEV_FLTA | DEV_DISABLE | DEV_DIS | DEV_UBUS | DEV_QBUS | DEV_DEBUG };
@@ -814,7 +888,7 @@ REG rqc_reg[] = {
 
 DEVICE rqc_dev = {
 	"RQC", rqc_unit, rqc_reg, rq_mod,
-	RQ_NUMDR + 2, DEV_RDX, 31, RQ_AINC, DEV_RDX, RQ_WID,
+	RQ_NUMDR + 2, DEV_RDX, 31, 2, DEV_RDX, 16,
 	NULL, NULL, &rq_reset,
 	&rq_boot, &rq_attach, &rq_detach,
 	&rqc_dib, DEV_FLTA | DEV_DISABLE | DEV_DIS | DEV_UBUS | DEV_QBUS | DEV_DEBUG };
@@ -878,7 +952,7 @@ REG rqd_reg[] = {
 
 DEVICE rqd_dev = {
 	"RQD", rqd_unit, rqd_reg, rq_mod,
-	RQ_NUMDR + 2, DEV_RDX, 31, RQ_AINC, DEV_RDX, RQ_WID,
+	RQ_NUMDR + 2, DEV_RDX, 31, 2, DEV_RDX, 16,
 	NULL, NULL, &rq_reset,
 	&rq_boot, &rq_attach, &rq_detach,
 	&rqd_dib, DEV_FLTA | DEV_DISABLE | DEV_DIS | DEV_UBUS | DEV_QBUS | DEV_DEBUG };
@@ -907,6 +981,8 @@ case 0:							/* IP */
 	*data = 0;					/* reads zero */
 	if (cp->csta == CST_S3_PPB) rq_step4 (cp);	/* waiting for poll? */
 	else if (cp->csta == CST_UP) {			/* if up */
+	    if (DEBUG_PRD (dptr)) fprintf (sim_deb,
+	        ">>RQ%c: poll started, PC=%X\n", 'A' + cp->cnum, OLDPC);
 	    cp->pip = 1;				/* poll host */
 	    sim_activate (dptr->units + RQ_QUEUE, rq_qtime);  }
 	break;
@@ -976,7 +1052,7 @@ else base = cp->comm + SA_COMM_CI;
 lnt = cp->comm + cp->cq.lnt + cp->rq.lnt - base;	/* comm lnt */
 if (lnt > SA_COMM_MAX) lnt = SA_COMM_MAX;		/* paranoia */
 for (i = 0; i < (lnt >> 1); i++) zero[i] = 0;		/* clr buffer */
-if (Map_WriteW (base, lnt, zero, MAP))			/* zero comm area */
+if (Map_WriteW (base, lnt, zero))			/* zero comm area */
 	return rq_fatal (cp, PE_QWE);			/* error? */
 cp->sa = SA_S4 | (RQ_UQPM << SA_S4C_V_MOD) |		/* send step 4 */
 	(RQ_SVER << SA_S4C_V_VER);
@@ -1426,8 +1502,7 @@ int32 rq_rw_valid (MSC *cp, int32 pkt, UNIT *uptr, uint32 cmd)
 uint32 dtyp = GET_DTYPE (uptr->flags);			/* get drive type */
 uint32 lbn = GETP32 (pkt, RW_LBNL);			/* get lbn */
 uint32 bc = GETP32 (pkt, RW_BCL);			/* get byte cnt */
-uint32 maxlbn = uptr->capac / RQ_NUMBY;			/* get max lbn */
-/* uint32 maxlbn = drv_tab[dtyp].lbn;			/* get max lbn */
+uint32 maxlbn = (uint32) (uptr->capac / RQ_NUMBY);	/* get max lbn */
 
 if ((uptr->flags & UNIT_ATT) == 0)			/* not attached? */
 	return (ST_OFL | SB_OFL_NV);			/* offl no vol */
@@ -1496,7 +1571,7 @@ if (cmd == OP_ERS) {					/* erase? */
 	err = ferror (uptr->fileref);  }		/* end if erase */
 
 else if (cmd == OP_WR) {				/* write? */
-	t = Map_ReadW (ba, tbc, rqxb, MAP);		/* fetch buffer */
+	t = Map_ReadW (ba, tbc, rqxb);			/* fetch buffer */
 	if (abc = tbc - t) {				/* any xfer? */
 	    wwc = ((abc + (RQ_NUMBY - 1)) & ~(RQ_NUMBY - 1)) >> 1;
 	    for (i = (abc >> 1); i < wwc; i++) rqxb[i] = 0;
@@ -1516,7 +1591,7 @@ else {	err = sim_fseek (uptr->fileref, da, SEEK_SET);	/* set pos */
 	    for ( ; i < (tbc >> 1); i++) rqxb[i] = 0;	/* fill */
 	    err = ferror (uptr->fileref);  }
 	if ((cmd == OP_RD) && !err) {			/* read? */
-	    if (t = Map_WriteW (ba, tbc, rqxb, MAP)) {	/* store, nxm? */
+	    if (t = Map_WriteW (ba, tbc, rqxb)) {	/* store, nxm? */
 		PUTP32 (pkt, RW_WBCL, bc - (tbc - t));	/* adj bc */
 		PUTP32 (pkt, RW_WBAL, ba + (tbc - t));	/* adj ba */
 		if (rq_hbe (cp, uptr))			/* post err log */
@@ -1526,7 +1601,7 @@ else {	err = sim_fseek (uptr->fileref, da, SEEK_SET);	/* set pos */
 	else if ((cmd == OP_CMP) && !err) {		/* compare? */
 	    uint8 dby, mby;
 	    for (i = 0; i < tbc; i++) {			/* loop */
-		if (Map_ReadB (ba + i, 1, &mby, MAP)) {	/* fetch, nxm? */
+		if (Map_ReadB (ba + i, 1, &mby)) {	/* fetch, nxm? */
 		    PUTP32 (pkt, RW_WBCL, bc - i);	/* adj bc */
 		    PUTP32 (pkt, RW_WBAL, bc - i);	/* adj ba */
 		    if (rq_hbe (cp, uptr))		/* post err log */
@@ -1761,7 +1836,7 @@ if ((desc & UQ_DESC_OWN) == 0) {			/* none */
 if (!rq_deqf (cp, pkt)) return ERR;			/* get cmd pkt */
 cp->hat = 0;						/* dsbl hst timer */
 addr = desc & UQ_ADDR;					/* get Q22 addr */
-if (Map_ReadW (addr + UQ_HDR_OFF, RQ_PKT_SIZE, cp->pak[*pkt].d, MAP))
+if (Map_ReadW (addr + UQ_HDR_OFF, RQ_PKT_SIZE, cp->pak[*pkt].d))
 	return rq_fatal (cp, PE_PRE);			/* read pkt */
 return rq_putdesc (cp, &cp->cq, desc);			/* release desc */
 }
@@ -1793,7 +1868,7 @@ if ((GETP (pkt, UQ_HCTC, TYP) == UQ_TYP_SEQ) &&		/* seq packet? */
 	cr = (cp->credits >= 14)? 14: cp->credits;	/* max 14 credits */
 	cp->credits = cp->credits - cr;			/* decr credits */
 	cp->pak[pkt].d[UQ_HCTC] |= ((cr + 1) << UQ_HCTC_V_CR);  }
-if (Map_WriteW (addr + UQ_HDR_OFF, lnt, cp->pak[pkt].d, MAP))
+if (Map_WriteW (addr + UQ_HDR_OFF, lnt, cp->pak[pkt].d))
 	return rq_fatal (cp, PE_PWE);			/* write pkt */
 rq_enqh (cp, &cp->freq, pkt);				/* pkt is free */
 cp->pbsy = cp->pbsy - 1;				/* decr busy cnt */
@@ -1808,7 +1883,7 @@ t_bool rq_getdesc (MSC *cp, struct uq_ring *ring, uint32 *desc)
 uint32 addr = ring->ba + ring->idx;
 uint16 d[2];
 
-if (Map_ReadW (addr, 4, d, MAP))			/* fetch desc */
+if (Map_ReadW (addr, 4, d))				/* fetch desc */
 	return rq_fatal (cp, PE_QRE);			/* err? dead */
 *desc = ((uint32) d[0]) | (((uint32) d[1]) << 16);
 return OK;
@@ -1828,13 +1903,13 @@ uint16 d[2];
 
 d[0] = newd & 0xFFFF;					/* 32b to 16b */
 d[1] = (newd >> 16) & 0xFFFF;
-if (Map_WriteW (addr, 4, d, MAP))			/* store desc */
+if (Map_WriteW (addr, 4, d))				/* store desc */
 	return rq_fatal (cp, PE_QWE);			/* err? dead */
 if (desc & UQ_DESC_F) {					/* was F set? */
 	if (ring->lnt <= 4) rq_ring_int (cp, ring);	/* lnt = 1? intr */
 	else {						/* prv desc */
 	    prva = ring->ba + ((ring->idx - 4) & (ring->lnt - 1));
-	    if (Map_ReadW (prva, 4, d, MAP))		/* read prv */
+	    if (Map_ReadW (prva, 4, d))			/* read prv */
 		return rq_fatal (cp, PE_QRE);
 	    prvd = ((uint32) d[0]) | (((uint32) d[1]) << 16);
 	    if (prvd & UQ_DESC_OWN) rq_ring_int (cp, ring);  }  }
@@ -1872,7 +1947,7 @@ return;
 void rq_putr_unit (MSC *cp, int32 pkt, UNIT *uptr, uint32 lu, t_bool all)
 {
 uint32 dtyp = GET_DTYPE (uptr->flags);			/* get drive type */
-uint32 maxlbn = uptr->capac / RQ_NUMBY;			/* get max lbn */
+uint32 maxlbn = (uint32) (uptr->capac / RQ_NUMBY);	/* get max lbn */
 
 cp->pak[pkt].d[ONL_MLUN] = lu;				/* unit */
 cp->pak[pkt].d[ONL_UFL] = uptr->uf | UF_RPL | RQ_WPH (uptr) | RQ_RMV (uptr);
@@ -1921,7 +1996,7 @@ void rq_ring_int (MSC *cp, struct uq_ring *ring)
 uint32 iadr = cp->comm + ring->ioff;			/* addr intr wd */
 uint16 flag = 1;
 
-Map_WriteW (iadr, 2, &flag, MAP);			/* write flag */
+Map_WriteW (iadr, 2, &flag);				/* write flag */
 if (cp->s1dat & SA_S1H_VEC) rq_setint (cp); 		/* if enb, intr */
 return;
 }
@@ -2016,13 +2091,14 @@ uint32 cap;
 uint32 max = sim_taddr_64? RA8U_EMAXC: RA8U_MAXC;
 t_stat r;
 
-if ((val < 0) || (val > RA8U_DTYPE) || ((val != RA8U_DTYPE) && cptr))
+if ((val < 0) || ((val != RA8U_DTYPE) && cptr))
 	return SCPE_ARG;
 if (uptr->flags & UNIT_ATT) return SCPE_ALATT;
 if (cptr) {
-	cap = (int32) get_uint (cptr, 10, max, &r);
-	if ((r != SCPE_OK) || (cap < RA8U_MINC)) return SCPE_ARG;
-	drv_tab[val].lbn = cap * 1954;  }
+	cap = (uint32) get_uint (cptr, 10, 0xFFFFFFFF, &r);
+	if ((sim_switches & SWMASK ('L')) == 0) cap = cap * 1954;
+	if ((r != SCPE_OK) || (cap < RA8U_MINC) || (cap >= max)) return SCPE_ARG;
+	drv_tab[val].lbn = cap;  }
 uptr->flags = (uptr->flags & ~UNIT_DTYPE) | (val << UNIT_V_DTYPE);
 uptr->capac = ((t_addr) drv_tab[val].lbn) * RQ_NUMBY;
 return SCPE_OK;
@@ -2077,12 +2153,19 @@ for (i = 0, cidx = -1; i < RQ_NUMCT; i++) {		/* find ctrl num */
 if (cidx < 0) return SCPE_IERR;				/* not found??? */
 cp = rq_ctxmap[cidx];					/* get context */
 cp->cnum = cidx;					/* init index */
+
+#if defined (VM_VAX)					/* VAX */
+cp->ubase = 0;						/* unit base = 0 */
+#else							/* PDP-11 */
 cp->ubase = cidx * RQ_NUMDR;				/* init unit base */
+#endif
+
 cp->csta = CST_S1;					/* init stage 1 */
 cp->s1dat = 0;						/* no S1 data */
 dibp->vec = 0;						/* no vector */
 cp->comm = 0;						/* no comm region */
-cp->sa = SA_S1 | SA_S1C_Q22 | SA_S1C_DI | SA_S1C_MP;	/* init SA val */
+if (UNIBUS) cp->sa = SA_S1 | SA_S1C_DI | SA_S1C_MP;	/* Unibus? */
+else cp->sa = SA_S1 | SA_S1C_Q22 | SA_S1C_DI | SA_S1C_MP; /* init SA val */
 cp->cflgs = CF_RPL;					/* ctrl flgs off */
 cp->htmo = RQ_DHTMO;					/* default timeout */
 cp->hat = cp->htmo;					/* default timer */
@@ -2105,7 +2188,7 @@ for (i = 0; i < (RQ_NUMDR + 2); i++) {			/* init units */
 	uptr->flags = uptr->flags & ~(UNIT_ONL | UNIT_ATP);
 	uptr->uf = 0;					/* clr unit flags */
 	uptr->cpkt = uptr->pktq = 0;  }			/* clr pkt q's */
-if (rqxb == NULL) rqxb = calloc (RQ_MAXFR >> 1, sizeof (unsigned int16));
+if (rqxb == NULL) rqxb = calloc (RQ_MAXFR >> 1, sizeof (uint16));
 if (rqxb == NULL) return SCPE_MEM;
 return auto_config (0, 0);				/* run autoconfig */
 }
@@ -2223,7 +2306,7 @@ fprintf (st, "ring, base = %x, index = %d, length = %d\n",
 	 rp->ba, rp->idx >> 2, rp->lnt >> 2);
 #endif
 for (i = 0; i < (rp->lnt >> 2); i++) {
-	if (Map_ReadW (rp->ba + (i << 2), 4, d, MAP)) {
+	if (Map_ReadW (rp->ba + (i << 2), 4, d)) {
 		fprintf (st, " %3d: non-existent memory\n", i);
 		break;  }
 	desc = ((uint32) d[0]) | (((uint32) d[1]) << 16);
